EXPLAIN EXTENDED
SELECT id, a FROM t1 WHERE a;
SELECT id, a FROM t1 WHERE a;

EXPLAIN EXTENDED
SELECT id, a FROM t1 WHERE NOT a;
SELECT id, a FROM t1 WHERE NOT a;

EXPLAIN EXTENDED
SELECT id, a FROM t1 WHERE a IS TRUE;
SELECT id, a FROM t1 WHERE a IS TRUE;

EXPLAIN EXTENDED
SELECT id, a FROM t1 WHERE NOT a IS TRUE;
SELECT id, a FROM t1 WHERE NOT a IS TRUE;

EXPLAIN EXTENDED
SELECT id, a FROM t1 WHERE a IS NOT TRUE;
SELECT id, a FROM t1 WHERE a IS NOT TRUE;

EXPLAIN EXTENDED
SELECT id, a FROM t1 WHERE NOT a IS NOT TRUE;
SELECT id, a FROM t1 WHERE NOT a IS NOT TRUE;

EXPLAIN EXTENDED
SELECT id, a FROM t1 WHERE a IS FALSE;
SELECT id, a FROM t1 WHERE a IS FALSE;

EXPLAIN EXTENDED
SELECT id, a FROM t1 WHERE NOT a IS FALSE;
SELECT id, a FROM t1 WHERE NOT a IS FALSE;

EXPLAIN EXTENDED
SELECT id, a FROM t1 WHERE a IS NOT FALSE;
SELECT id, a FROM t1 WHERE a IS NOT FALSE;

EXPLAIN EXTENDED
SELECT id, a FROM t1 WHERE NOT a IS NOT FALSE;
SELECT id, a FROM t1 WHERE NOT a IS NOT FALSE;
